<?php
namespace Drupal\transformation_activation\Plugin\rest\resource;

use Drupal\transformation_activation\Orders;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\rest\ModifiedResourceResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * @RestResource(
 *   id = "add_register",
 *   label = @Translation("añadir/consultar registro"),
 *   uri_paths = {
 *     "canonical" = "/api/seller/query",
 *     "https://www.drupal.org/link-relations/create" = "/api/seller/add"
 *   }
 * )
 */
class AddRegisterRestResource extends ResourceBase {

  /**
   * {@inheritdoc}
   */
  public function get() {
    \Drupal::service('page_cache_kill_switch')->trigger();
    $authentication = $this->getToken();

    if(empty($authentication)) {
      $response = [
        'code' => 401,
        'status' => $this->t('Authorization required.'),
      ];
      return new ResourceResponse($response, 401);
    }

    $service = \Drupal::service('tigo_seller_auth.access_token_api');

    if($service->validateSellerAccessToken($authentication)) {
      $get = \Drupal::request()->query->all();
      $response = [
        'code' => 200,
        'status' => '',
      ];

      if($get['seller_id'] || $get['order_id']) {
        try {
          $database = \Drupal::database();
          $query = $database->select('vtex_orders', 'vo')
            ->fields('vo', [
              'order_id',
              'item_id',
              'quantity_id',
              'json_address',
              'client_name',
              'client_document',
              'client_phone',
              'deliver',
              'client_email',
              'creation_date',
              'quantity',
              'description',
              'ref_id',
              'id',
              'product_id',
              'selling_price',
              'payment_name',
              'status',
              'activation_status',
              'total_value',
              'send',
              'imsi',
              'msisdn',
              'dealer_id',
              'dealer_info',
            ]);

          if($get['seller_id']) {
            $query->condition('seller_id', $get['seller_id']);
          }

          if($get['order_id']) {
            $query->condition('order_id', $get['order_id']);
          }

          if(isset($get['init_date']) && isset($get['end_date']) && isset($get['seller_id'])) {
            $query->condition('creation_date', [$get['init_date'], $get['end_date']], 'BETWEEN');
          }

          $db_data = $query->execute()->fetchAll();

          if(empty($db_data)) {
            $response['status'] = $this->t('Data not found');
          }
          else {
            foreach($db_data as $value) {
              $response['data'][] = (array) $value;
            }
          }

        }
        catch(\Exception $e) {
          $response['code'] = 500;
          $response['status'] = $this->t('Internal server error.');
          unset($response['data']);
        }
      }
    }
    else {
      $response['code'] = 401;
      $response['status'] = $this->t('Invalid access token.');
    }

    return new ResourceResponse($response, $response['code']);
  }

  /**
   * {@inheritdoc}
   */
  public function post($data) {
    \Drupal::service('page_cache_kill_switch')->trigger();

    $authentication = $this->getToken();

    if(empty($authentication)) {
      $response = [
        'code' => 401,
        'status' => $this->t('Authorization required.'),
      ];
      return new ModifiedResourceResponse($response, 401);
    }

    $service = \Drupal::service('tigo_seller_auth.access_token_api');

    if($service->validateSellerAccessToken($authentication)) {
      $required_fields = [
        'order_id',
        'dealer_id',
        'client_name',
        'client_email',
        'client_document',
        'client_phone',
        'quantity',
        'ref_id',
        'selling_price',
        'creation_date',
        'description',
        'sequence',
        'payment_name'
      ];

      $order = new Orders($data, $required_fields);

      $response = [];

      try {
        $no_fields = $order->verifyRequiredFields();

        if(empty($no_fields) != 1) {
          $fields = "";
          foreach($no_fields as $field) {
            $fields .= $field . ' ';
          }
          $fields = trim($fields);
          throw new \Exception("This fields are required: $fields");
        }

        if(strlen($order->get('order_id')) > 16) {
          throw new \Exception('Order_id incorrecto, debe tener menos o 16 caracteres.');
        }

        $verify = $order->validateFields();

        if(!empty($verify)) {
          throw new \Exception($verify['message']);
        }

        $response = $order->pushOrder();

      }
      catch(\Exception $e) {
        $response = [
          'code' => 500,
          'message' => $e->getMessage(),
          //'sqlerror' => $e->getCode(),
        ];
      }
    }
    else {
      $response['code'] = 401;
      $response['status'] = $this->t('Invalid access token.');
    }

    return new ModifiedResourceResponse($response, $response['code']);
  }

  /**
   * @return string
   */
  public function getToken() {
    $authorization = \Drupal::request()->headers->get('Authorization');
    return trim(str_replace('Bearer', '', $authorization));
  }

}